package xyz.leomurca.csgomatches.ui.components

import androidx.compose.foundation.background
import androidx.compose.foundation.layout.Arrangement
import androidx.compose.foundation.layout.Box
import androidx.compose.foundation.layout.BoxScope
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.Row
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.height
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.layout.requiredWidthIn
import androidx.compose.foundation.layout.size
import androidx.compose.foundation.layout.wrapContentWidth
import androidx.compose.foundation.shape.RoundedCornerShape
import androidx.compose.material3.Card
import androidx.compose.material3.CardDefaults
import androidx.compose.material3.MaterialTheme
import androidx.compose.material3.Text
import androidx.compose.runtime.Composable
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.draw.clip
import androidx.compose.ui.draw.drawBehind
import androidx.compose.ui.geometry.Offset
import androidx.compose.ui.graphics.Color
import androidx.compose.ui.layout.ContentScale
import androidx.compose.ui.platform.LocalContext
import androidx.compose.ui.res.painterResource
import androidx.compose.ui.res.stringResource
import androidx.compose.ui.text.style.TextAlign
import androidx.compose.ui.unit.Dp
import androidx.compose.ui.unit.dp
import coil3.compose.AsyncImage
import xyz.leomurca.csgomatches.R
import xyz.leomurca.csgomatches.domain.model.League
import xyz.leomurca.csgomatches.domain.model.Match
import xyz.leomurca.csgomatches.domain.model.MatchStatus
import xyz.leomurca.csgomatches.domain.model.Opponent
import xyz.leomurca.csgomatches.ui.navigation.MatchDetailsRoute
import xyz.leomurca.csgomatches.ui.theme.LiveRed
import xyz.leomurca.csgomatches.ui.theme.White_20
import xyz.leomurca.csgomatches.ui.theme.White_50
import xyz.leomurca.csgomatches.utils.toFormattedMatchTime
import java.time.ZonedDateTime

@Composable
fun MatchCard(
    match: Match,
    onTapCard: (MatchDetailsRoute) -> Unit,
    modifier: Modifier = Modifier,
) {
    Box(modifier = modifier.fillMaxWidth()) {
        val (leftOpponent, rightOpponent) = getOrDefaultOpponents(match.opponents)
        val leagueAndSerieName = "${match.league.name} + ${match.serie.name}"
        val scheduleConfig = match.matchStatus.scheduleConfigFor(match.beginAt)

        Card(
            modifier = Modifier.fillMaxWidth(), shape = RoundedCornerShape(16.dp), onClick = {
                onTapCard(
                    MatchDetailsRoute(
                        leftOpponentId = leftOpponent.id,
                        rightOpponentId = rightOpponent.id,
                        leagueAndSerieName = leagueAndSerieName,
                        scheduleString = scheduleConfig.first,
                        matchStatus = match.matchStatus
                    )
                )
            }, colors = CardDefaults.cardColors(containerColor = MaterialTheme.colorScheme.surface)
        ) {
            MatchupRow(leftOpponent, rightOpponent)
            LeagueInfoRow(match.league, leagueAndSerieName)
        }
        ScheduleBadge(scheduleConfig)
    }
}

@Composable
private fun MatchupRow(leftOpponent: Opponent, rightOpponent: Opponent) {
    Row(
        modifier = Modifier
            .fillMaxWidth()
            .padding(top = 43.5.dp, bottom = 18.5.dp),
        horizontalArrangement = Arrangement.Center,
        verticalAlignment = Alignment.CenterVertically
    ) {

        Column(horizontalAlignment = Alignment.CenterHorizontally) {
            AsyncImage(
                model = leftOpponent.imageUrl,
                contentDescription = stringResource(R.string.logo_description, leftOpponent.name),
                error = painterResource(R.drawable.fallback_image_round),
                placeholder = painterResource(R.drawable.fallback_image_round),
                modifier = Modifier.size(60.dp),
                contentScale = ContentScale.Fit
            )
            Text(
                leftOpponent.name,
                style = MaterialTheme.typography.labelMedium,
                textAlign = TextAlign.Center,
                modifier = Modifier
                    .padding(top = 10.dp)
                    .requiredWidthIn(max = 60.dp)
            )
        }

        Text(
            stringResource(R.string.versus),
            style = MaterialTheme.typography.bodyMedium,
            color = White_50,
            modifier = Modifier.padding(horizontal = 20.dp)
        )

        Column(horizontalAlignment = Alignment.CenterHorizontally) {
            AsyncImage(
                model = rightOpponent.imageUrl,
                contentDescription = stringResource(R.string.logo_description, rightOpponent.name),
                error = painterResource(R.drawable.fallback_image_round),
                placeholder = painterResource(R.drawable.fallback_image_round),
                modifier = Modifier.size(60.dp),
                contentScale = ContentScale.Fit
            )
            Text(
                rightOpponent.name,
                style = MaterialTheme.typography.labelMedium,
                textAlign = TextAlign.Center,
                modifier = Modifier
                    .padding(top = 10.dp)
                    .requiredWidthIn(max = 60.dp)
            )
        }
    }
}

@Composable
private fun LeagueInfoRow(league: League, leagueAndSerieName: String) {
    Row(
        verticalAlignment = Alignment.CenterVertically,
        modifier = Modifier
            .fillMaxWidth()
            .topBorder(White_20, 1.dp)
            .padding(vertical = 8.dp, horizontal = 16.dp)
    ) {
        AsyncImage(
            model = league.imageUrl,
            contentDescription = stringResource(R.string.logo_description, league.name),
            error = painterResource(R.drawable.fallback_image_round),
            placeholder = painterResource(R.drawable.fallback_image_round),
            modifier = Modifier
                .height(16.dp)
                .wrapContentWidth(),
            contentScale = ContentScale.Fit
        )
        Text(
            leagueAndSerieName,
            style = MaterialTheme.typography.labelSmall,
            modifier = Modifier.padding(start = 8.dp)
        )
    }
}

@Composable
private fun BoxScope.ScheduleBadge(scheduleConfig: Pair<String, Color>) {
    val (scheduleString, scheduleBackgroundColor) = scheduleConfig
    Box(
        modifier = Modifier
            .align(Alignment.TopEnd)
            .clip(RoundedCornerShape(topEnd = 16.dp, bottomStart = 16.dp))
            .background(scheduleBackgroundColor)
            .padding(8.dp)
    ) {
        Text(
            text = scheduleString,
            style = MaterialTheme.typography.displayMedium,
            color = Color.White
        )
    }
}

private fun Modifier.topBorder(color: Color, thickness: Dp): Modifier = this.then(
    Modifier.drawBehind {
        val strokeWidth = thickness.toPx()
        drawLine(
            color = color,
            start = Offset(0f, 0f),
            end = Offset(size.width, 0f),
            strokeWidth = strokeWidth
        )
    })

@Composable
private fun getOrDefaultOpponents(opponents: List<Opponent>): Pair<Opponent, Opponent> {
    val default = Opponent(id = -1, name = stringResource(R.string.to_be_defined), imageUrl = "")
    return when {
        opponents.size >= 2 -> opponents[0] to opponents[1]
        opponents.size == 1 -> opponents[0] to default
        else -> default to default
    }
}

@Composable
private fun MatchStatus.scheduleConfigFor(beginAt: ZonedDateTime?) = when (this) {
    MatchStatus.LIVE -> stringResource(R.string.live) to LiveRed
    MatchStatus.SCHEDULED -> beginAt.toFormattedMatchTime(LocalContext.current) to White_20
    MatchStatus.UNKNOWN -> stringResource(R.string.to_be_defined) to White_20
}
