package xyz.leomurca.csgomatches.ui.screens.matchdetails

import androidx.compose.foundation.background
import androidx.compose.foundation.layout.Arrangement
import androidx.compose.foundation.layout.Box
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.Row
import androidx.compose.foundation.layout.Spacer
import androidx.compose.foundation.layout.absoluteOffset
import androidx.compose.foundation.layout.fillMaxSize
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.layout.requiredWidthIn
import androidx.compose.foundation.layout.size
import androidx.compose.foundation.layout.width
import androidx.compose.foundation.lazy.LazyColumn
import androidx.compose.foundation.lazy.items
import androidx.compose.foundation.shape.RoundedCornerShape
import androidx.compose.material.icons.Icons
import androidx.compose.material.icons.automirrored.filled.ArrowBack
import androidx.compose.material3.CircularProgressIndicator
import androidx.compose.material3.Icon
import androidx.compose.material3.IconButton
import androidx.compose.material3.MaterialTheme
import androidx.compose.material3.Text
import androidx.compose.runtime.Composable
import androidx.compose.runtime.LaunchedEffect
import androidx.compose.runtime.collectAsState
import androidx.compose.runtime.getValue
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.remember
import androidx.compose.runtime.setValue
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.draw.clip
import androidx.compose.ui.graphics.Color
import androidx.compose.ui.layout.ContentScale
import androidx.compose.ui.res.painterResource
import androidx.compose.ui.res.stringResource
import androidx.compose.ui.text.style.TextAlign
import androidx.compose.ui.text.style.TextOverflow
import androidx.compose.ui.unit.Dp
import androidx.compose.ui.unit.dp
import androidx.hilt.navigation.compose.hiltViewModel
import coil3.compose.AsyncImage
import coil3.compose.AsyncImagePainter
import xyz.leomurca.csgomatches.R
import xyz.leomurca.csgomatches.domain.model.MatchStatus
import xyz.leomurca.csgomatches.domain.model.Player
import xyz.leomurca.csgomatches.domain.model.Team
import xyz.leomurca.csgomatches.ui.components.ErrorMessage
import xyz.leomurca.csgomatches.ui.components.LoadingIndicator
import xyz.leomurca.csgomatches.ui.navigation.MatchDetailsRoute
import xyz.leomurca.csgomatches.ui.theme.LiveRed
import xyz.leomurca.csgomatches.ui.theme.White
import xyz.leomurca.csgomatches.ui.theme.White_50

@Composable
fun MatchDetailsScreen(
    viewModel: MatchDetailsViewModel = hiltViewModel(),
    matchDetails: MatchDetailsRoute,
    onBackClick: () -> Unit
) {
    val uiState = viewModel.uiState.collectAsState()

    LaunchedEffect(Unit) {
        loadTeams(matchDetails, viewModel)
    }

    Column(
        modifier = Modifier
            .fillMaxSize()
            .background(MaterialTheme.colorScheme.background),
    ) {
        val value = uiState.value
        when {
            value.isLoading -> LoadingIndicator()
            value.errorMessage != null -> ErrorMessage(
                message = value.errorMessage,
                onRetry = { loadTeams(matchDetails, viewModel) }
            )

            value.leftTeam != null && value.rightTeam != null -> Column {
                TopBar(matchDetails.leagueAndSerieName, onBackClick)
                MatchupRow(leftTeam = value.leftTeam, rightTeam = value.rightTeam)
                ScheduleRow(matchDetails.scheduleString, matchDetails.matchStatus)
                DualPlayerLists(value.leftTeam.players, value.rightTeam.players)
            }
        }
    }
}

@Composable
private fun DualPlayerLists(
    leftPlayers: List<Player>,
    rightPlayers: List<Player>,
    modifier: Modifier = Modifier
) {
    Row(
        modifier = modifier
            .fillMaxWidth()
            .padding(start = 0.dp, top = 16.dp, end = 0.dp, bottom = 60.dp)
    ) {
        // Left team
        LazyColumn(
            modifier = Modifier
                .weight(1f)
                .padding(end = 4.dp)
        ) {
            items(leftPlayers) { player ->
                PlayerCard(player = player, isLeft = true)
            }
        }

        // Right team
        LazyColumn(
            modifier = Modifier
                .weight(1f)
                .padding(start = 4.dp)
        ) {
            items(rightPlayers) { player ->
                PlayerCard(player = player, isLeft = false)
            }
        }
    }
}

@Composable
private fun PlayerCard(player: Player, isLeft: Boolean) {
    val roundedCornerShape = if (isLeft) RoundedCornerShape(topEnd = 12.dp, bottomEnd = 12.dp)
    else RoundedCornerShape(topStart = 12.dp, bottomStart = 12.dp)

    Row(
        modifier = Modifier
            .fillMaxWidth()
            .padding(vertical = 6.dp)
            .background(MaterialTheme.colorScheme.surface, roundedCornerShape)
            .padding(
                top = 8.dp,
                bottom = 8.dp,
                start = if (isLeft) 4.dp else 12.dp,
                end = if (isLeft) 12.dp else 4.dp
            ),
        verticalAlignment = Alignment.CenterVertically,
        horizontalArrangement = if (isLeft) Arrangement.End else Arrangement.Start
    ) {
        if (isLeft) {
            PlayerInfo(player, Modifier.width(100.dp), alignEnd = true)
            Spacer(modifier = Modifier.width(12.dp))
            PlayerAvatar(size = 48.dp, player.imageUrl, player.nickName)
        } else {
            PlayerAvatar(size = 48.dp, player.imageUrl, player.nickName)
            Spacer(modifier = Modifier.width(12.dp))
            PlayerInfo(player, Modifier.width(100.dp), alignEnd = false)
        }
    }
}

@Composable
private fun PlayerAvatar(size: Dp, imageUrl: String?, nickName: String?) {
    var isLoading by remember { mutableStateOf(true) }
    Box(
        modifier = Modifier
            .absoluteOffset(y = (-12).dp)
            .size(size)
            .clip(RoundedCornerShape(8.dp))
            .background(Color.LightGray)
    ) {
        AsyncImage(
            model = imageUrl,
            contentDescription = stringResource(R.string.logo_description, nickName ?: ""),
            contentScale = ContentScale.Crop,
            onState = { isLoading = it is AsyncImagePainter.State.Loading },
            modifier = Modifier.fillMaxSize()
        )
        if (isLoading) {
            CircularProgressIndicator(
                modifier = Modifier
                    .size(16.dp)
                    .align(Alignment.Center),
                strokeWidth = 2.dp,
                color = MaterialTheme.colorScheme.background
            )
        }
    }
}

@Composable
private fun PlayerInfo(player: Player, modifier: Modifier = Modifier, alignEnd: Boolean) {
    Column(
        modifier = modifier,
        horizontalAlignment = if (alignEnd) Alignment.End else Alignment.Start
    ) {
        Text(
            text = player.nickName ?: stringResource(R.string.to_be_defined),
            color = Color.White,
            style = MaterialTheme.typography.headlineLarge,
            maxLines = 1,
            overflow = TextOverflow.Ellipsis
        )
        Text(
            text = playerFullNameOrEmpty(player.firstName, player.lastName),
            color = Color.Gray,
            style = MaterialTheme.typography.bodySmall,
            maxLines = 1,
            overflow = TextOverflow.Ellipsis
        )
    }
}

@Composable
private fun ScheduleRow(scheduleString: String, matchStatus: MatchStatus) {
    Row(
        Modifier
            .fillMaxWidth()
            .padding(top = 20.dp), horizontalArrangement = Arrangement.Center
    ) {
        Text(
            scheduleString,
            style = MaterialTheme.typography.headlineMedium,
            color = White,
            modifier = if (matchStatus == MatchStatus.LIVE)
                Modifier
                    .background(LiveRed, RoundedCornerShape(16.dp))
                    .padding(horizontal = 12.dp, vertical = 8.dp)
            else Modifier
        )
    }
}

@Composable
private fun MatchupRow(leftTeam: Team, rightTeam: Team) {
    Row(
        modifier = Modifier
            .fillMaxWidth()
            .padding(top = 24.dp, bottom = 20.dp),
        horizontalArrangement = Arrangement.Center,
        verticalAlignment = Alignment.CenterVertically
    ) {
        Column(horizontalAlignment = Alignment.CenterHorizontally) {
            AsyncImage(
                model = leftTeam.imageUrl,
                contentDescription = stringResource(R.string.logo_description, leftTeam.name ?: ""),
                error = painterResource(R.drawable.fallback_image_round),
                placeholder = painterResource(R.drawable.fallback_image_round),
                modifier = Modifier.size(60.dp),
                contentScale = ContentScale.Fit
            )
            Text(
                leftTeam.name ?: stringResource(R.string.to_be_defined),
                style = MaterialTheme.typography.labelMedium,
                color = White,
                textAlign = TextAlign.Center,
                modifier = Modifier
                    .padding(top = 10.dp)
                    .requiredWidthIn(max = 60.dp)
            )
        }
        Text(
            stringResource(R.string.versus),
            style = MaterialTheme.typography.bodyMedium,
            color = White_50,
            modifier = Modifier.padding(horizontal = 20.dp)
        )

        Column(horizontalAlignment = Alignment.CenterHorizontally) {
            AsyncImage(
                model = rightTeam.imageUrl,
                contentDescription = stringResource(
                    R.string.logo_description,
                    rightTeam.name ?: ""
                ),
                error = painterResource(R.drawable.fallback_image_round),
                placeholder = painterResource(R.drawable.fallback_image_round),
                modifier = Modifier.size(60.dp),
                contentScale = ContentScale.Fit
            )
            Text(
                rightTeam.name ?: stringResource(R.string.to_be_defined),
                style = MaterialTheme.typography.labelMedium,
                color = White,
                textAlign = TextAlign.Center,
                modifier = Modifier
                    .padding(top = 10.dp)
                    .requiredWidthIn(max = 60.dp)
            )
        }
    }
}

@Composable
private fun TopBar(leagueAndSerieName: String, onBackClick: () -> Unit) {
    Box(
        modifier = Modifier
            .fillMaxWidth()
            .background(MaterialTheme.colorScheme.background)
            .padding(top = 52.dp)
    ) {
        IconButton(
            onClick = onBackClick,
            modifier = Modifier
                .align(Alignment.CenterStart)
                .padding(start = 18.dp)
        ) {
            Icon(
                imageVector = Icons.AutoMirrored.Default.ArrowBack,
                contentDescription = stringResource(R.string.back),
                tint = White
            )
        }

        Text(
            text = leagueAndSerieName,
            modifier = Modifier
                .align(Alignment.Center)
                .requiredWidthIn(max = 250.dp),
            textAlign = TextAlign.Center,
            style = MaterialTheme.typography.titleMedium,
            color = White,
        )
    }
}

private fun playerFullNameOrEmpty(firstName: String?, lastName: String?): String {
    val fullName = listOfNotNull(firstName?.trim(), lastName?.trim())
        .filter { it.isNotBlank() }
        .joinToString(" ")

    return if (fullName.isBlank()) "" else fullName
}

private fun loadTeams(matchDetails: MatchDetailsRoute, viewModel: MatchDetailsViewModel) {
    loadTeamOrPlaceholder(matchDetails.leftOpponentId, isLeft = true, viewModel)
    loadTeamOrPlaceholder(matchDetails.rightOpponentId, isLeft = false, viewModel)
}

private fun loadTeamOrPlaceholder(
    opponentId: Long,
    isLeft: Boolean,
    viewModel: MatchDetailsViewModel
) {
    if (opponentId != -1L) {
        viewModel.loadTeam(opponentId.toString(), isLeft)
    } else {
        viewModel.applyPlaceholderTeamToSide(isLeft)
    }
}
