package xyz.leomurca.csgomatches.ui.screens.matchdetails

import androidx.lifecycle.ViewModel
import androidx.lifecycle.viewModelScope
import dagger.hilt.android.lifecycle.HiltViewModel
import kotlinx.coroutines.flow.MutableStateFlow
import kotlinx.coroutines.flow.StateFlow
import kotlinx.coroutines.flow.asStateFlow
import kotlinx.coroutines.flow.update
import kotlinx.coroutines.launch
import xyz.leomurca.csgomatches.domain.model.Player
import xyz.leomurca.csgomatches.domain.model.Resource
import xyz.leomurca.csgomatches.domain.model.Team
import xyz.leomurca.csgomatches.domain.repository.MatchRepository
import javax.inject.Inject
import kotlin.String

@HiltViewModel
class MatchDetailsViewModel @Inject constructor(
    private val matchRepository: MatchRepository
) : ViewModel() {

    private val _uiState = MutableStateFlow(TeamUiState())
    val uiState: StateFlow<TeamUiState> = _uiState.asStateFlow()

    fun loadTeam(teamId: String, isLeft: Boolean) {
        viewModelScope.launch {
            setLoading()
            when (val result = matchRepository.teamDetails(teamId)) {
                is Resource.Success -> {
                    val team = teamOrWithDefaultPlayers(result.data.firstOrNull())
                    updateTeam(team, isLeft)
                }

                is Resource.Error -> {
                    _uiState.update {
                        it.copy(errorMessage = result.message, isLoading = false)
                    }
                }
            }
        }
    }

    fun applyPlaceholderTeamToSide(isLeft: Boolean) {
        setLoading()
        updateTeam(teamPlaceholder(), isLeft)
    }

    private fun teamOrWithDefaultPlayers(team: Team?): Team? {
        return team?.let {
            val filledPlayers =
                it.players + List((DEFAULT_TEAM_SIZE - it.players.size).coerceAtLeast(0)) {
                    playerPlaceholder()
                }
            it.copy(players = filledPlayers)
        }
    }

    private fun updateTeam(team: Team?, isLeft: Boolean) {
        _uiState.update {
            if (isLeft) it.copy(leftTeam = team, isLoading = false)
            else it.copy(rightTeam = team, isLoading = false)
        }
    }

    private fun setLoading() {
        _uiState.update { it.copy(isLoading = true, errorMessage = null) }
    }

    private fun playerPlaceholder() = Player(
        id = -1, nickName = null, firstName = null, lastName = null, imageUrl = null
    )

    private fun teamPlaceholder() = Team(
        id = -1, name = null, imageUrl = null, players = List(5) { playerPlaceholder() }
    )

    data class TeamUiState(
        val leftTeam: Team? = null,
        val rightTeam: Team? = null,
        val isLoading: Boolean = true,
        val errorMessage: String? = null
    )

    companion object {
        private const val DEFAULT_TEAM_SIZE = 5
    }
}