package xyz.leomurca.csgomatches.utils

import android.content.Context
import xyz.leomurca.csgomatches.R
import java.time.DayOfWeek
import java.time.LocalDate
import java.time.ZonedDateTime
import java.time.format.DateTimeFormatter
import java.util.Locale

/**
 * Formats a [ZonedDateTime] instance into a user-friendly, localized string representing the match time.
 *
 * The formatting rules are as follows:
 * - If the date is today, returns: `"Hoje, HH:mm"`
 * - If the date is tomorrow, returns: `"Amanhã, HH:mm"`
 * - If the date is within the current week, returns: `"EEE, HH:mm"` (e.g., `"Ter, 22:00"`)
 * - Otherwise, returns: `"dd.MM HH:mm"` (e.g., `"22.04 15:00"`)
 *
 * The day of the week is formatted in Portuguese (`pt-BR`) and its first letter is capitalized.
 *
 * @receiver the [ZonedDateTime] instance to format. If null, an empty string is returned.
 * @return a formatted string representing the match time, or `"A definir"` if the input is null.
 */
fun ZonedDateTime?.toFormattedMatchTime(
    context: Context,
    today: LocalDate = LocalDate.now()
): String {
    if (this == null) return context.getString(R.string.match_time_tbd)

    val targetDate = toLocalDate()
    val timeFormatter = DateTimeFormatter.ofPattern("HH:mm")

    return when {
        targetDate.isToday(today) -> context.getString(R.string.match_time_today, format(timeFormatter))
        targetDate.isTomorrow(today) -> context.getString(R.string.match_time_tomorrow, format(timeFormatter))
        targetDate.isInCurrentWeek(today) -> {
            val dayOfWeekFormatter = DateTimeFormatter.ofPattern("EEE", Locale("pt", "BR"))
            val day = format(dayOfWeekFormatter).replaceFirstChar {
                it.titlecase(Locale("pt", "BR"))
            }.dropLast(1)
            val time = format(timeFormatter)
            context.getString(R.string.match_time_weekday, day, time)
        }

        else -> {
            val fullFormatter = DateTimeFormatter.ofPattern("dd.MM HH:mm")
            format(fullFormatter)
        }
    }
}

private fun LocalDate.isToday(reference: LocalDate): Boolean = isEqual(reference)
private fun LocalDate.isTomorrow(reference: LocalDate): Boolean = isEqual(reference.plusDays(1))
private fun LocalDate.isInCurrentWeek(reference: LocalDate): Boolean {
    val startOfWeek = reference.with(DayOfWeek.MONDAY)
    val endOfWeek = reference.with(DayOfWeek.SUNDAY)
    return !isBefore(startOfWeek) && !isAfter(endOfWeek)
}