package xyz.leomurca.csgomatches.utils

import android.content.Context
import io.mockk.every
import io.mockk.mockk
import xyz.leomurca.csgomatches.R
import java.time.Clock
import java.time.DayOfWeek
import java.time.ZoneId
import java.time.ZonedDateTime
import java.time.format.DateTimeFormatter
import kotlin.test.Test
import kotlin.test.assertEquals

class ExtensionsTest {

    private val context: Context = mockk(relaxed = true)

    @Test
    fun `toFormattedMatchTime - returns Hoje when date is today`() {
        // Arrange
        val date = now().withHour(18).withMinute(30)
        every { context.getString(R.string.match_time_today, any()) } returns "Hoje, 18:30"

        // Act
        val result = date.toFormattedMatchTime(context, today = now().toLocalDate())

        // Assert
        assertEquals("Hoje, 18:30", result)
    }

    @Test
    fun `toFormattedMatchTime - returns Amanha when date is tomorrow`() {
        // Arrange
        val date = now().withHour(18).withMinute(30).plusDays(1)
        every { context.getString(R.string.match_time_tomorrow, any()) } returns "Amanhã, 18:30"

        // Act
        val result = date.toFormattedMatchTime(context, today = now().toLocalDate())

        // Assert
        assertEquals("Amanhã, 18:30", result)
    }

    @Test
    fun `toFormattedMatchTime - returns weekday in Portuguese when date is this week`() {
        // Arrange
        val tuesday = now().with(DayOfWeek.TUESDAY).withHour(22).withMinute(0)
        every { context.getString(R.string.match_time_weekday, any(), any()) } returns "Ter, 22:00"

        // Act
        val result = tuesday.toFormattedMatchTime(context, today = now().toLocalDate())

        // Assert
        assertEquals("Ter, 22:00", result)
    }

    @Test
    fun `toFormattedMatchTime - returns full date when not this week`() {
        // Arrange
        val future = now().plusWeeks(2).withHour(15).withMinute(0)

        // Act
        val result = future.toFormattedMatchTime(context, today = now().toLocalDate())

        // Assert
        val expected = future.format(DateTimeFormatter.ofPattern("dd.MM HH:mm"))
        assertEquals(expected, result)
    }

    @Test
    fun `toFormattedMatchTime - returns A definir when date is null`() {
        // Arrange
        val date: ZonedDateTime? = null
        every { context.getString(R.string.match_time_tbd) } returns "A definir"

        // Act
        val result = date.toFormattedMatchTime(context, today = now().toLocalDate())

        // Assert
        assertEquals("A definir", result)
    }

    private fun now(): ZonedDateTime = ZonedDateTime.now(fixedClock)

    companion object {
        private val fixedClock = Clock.fixed(
            ZonedDateTime.of(2025, 7, 19, 12, 0, 0, 0, ZoneId.of("America/Sao_Paulo")).toInstant(),
            ZoneId.of("America/Sao_Paulo")
        )
    }
}